import enum


class CommonStereotype(enum.Enum):
    IMPLEMENTATION = 1
    BIND = 2
    DERIVE = 3
    FRIEND = 4
    INSTANCEOF = 5
    INSTANTIATE = 6
    REFINE = 7
    USE = 8
    SUBSTITUTE = 9
    ACCESS = 10
    IMPORT = 11
    PERMIT = 12
    EXTEND = 13
    INCLUDE = 14
    BECOME = 15
    CALL = 16
    COPY = 17
    PARAMETER = 18
    SEND = 19


Stereotype = str
